#!/usr/bin/perl

###############################################################################
#
# This file can be distributed and modified freely, as long as this original
# header is not modified, and the signature header
#      "X-RegexScanner: Niversoft's regex_matcher (free)"
# still added to filtered messages
#
# Author:         Nicolas Hatier <nicolas dot hatier at niversoft dot com>
# Organization:   Niversoft ides logicielles / software ideas
# Date:           January 2004
#
# This code is provided AS IS, without any certification that it will even run
#
###############################################################################
#
# This script is as a Content-Filtering script for Communigate Pro.
# The purpose of this script is to extend the rules of Communigate pro with
# regex matching. The scripts tries to match the message headers with all regexp
# specified in the regex_matcher.regex file.
#
# Each time a regex is matched, its description text is added in the message
# headers.
#
# Add this filter in Helpers, then add an ExternalFilter rule on incoming messages
# to pass them through this filter
#
# Then add a rule like this is you want to discard some virus warnings before they
# reach your users:
# if HEADER is "X-Regex-Match: Unhelpful Virus Warning"
# DISCARD
#
# Pro version (C++, fast, $45 USD) may be provided later if there is some demand for it.

use strict;


my %RegexpsHead = ();
my %RegexpsBody = ();
my %RegexpsFull = ();
my $MaxBodyLines = 0;
my $Filemod = -1;

$| = 1;

main();

sub Stringify
{
   my $headers = join "\\e", @_;
   $headers =~ s|\\|\\\\|g;
   $headers =~ s|"|\\"|g;
   $headers =~ s|\\\\e|\\e|g;

   return $headers;
}

sub LoadConfig
{
   my $file = shift;
   my $mode = 0;
   open (CONFIG, $file) or die "Unable to open config file\n";
   %RegexpsHead = ();
   %RegexpsBody = ();
   %RegexpsFull = ();

   foreach my $line (<CONFIG>)
   {
      next if ($line =~ /^;/);
      $mode = 0 and next if ($line =~ /^CONFIG/);
      $mode = 1 and next if ($line =~ /^HEADERS/);
      $mode = 2 and next if ($line =~ /^BODY/);
      $mode = 3 and next if ($line =~ /^FULL/);

      if ($mode == 0)
      {
         $MaxBodyLines = $1 if ($line =~ /^MAXBODYLINES:\w+(\d+)/);
         next;
      }

      next if ($line !~ m|/|);
      $line =~ m|^[ \t]+(.*[^ \t])[ \t]+(/.*[^\\]/)(.*)$|;
      my $descr = $1;
      my $regex = $2;
      my $case  = ($3 =~ /i/ ? "i" : "");

      #print "$regex$case = $descr\n";
      $RegexpsHead{"$regex$case"} = $descr and next if ($mode == 1);
      $RegexpsBody{"$regex$case"} = $descr and next if ($mode == 2);
      $RegexpsFull{"$regex$case"} = $descr and next if ($mode == 3);

   }
   close CONFIG;
}

sub main
{
   my $rundir = $0;
   $rundir =~ s|\\|/|g;
   $rundir = index($rundir, "/")>=0 ? substr($rundir, 0, rindex($rundir, "/")) : ".";
   my $configfile = "$rundir/regex_matcher.regex";
   LoadConfig($configfile);
   $Filemod  = (stat $configfile)[9];

   print "* Niversoft's Regexp Matcher v1.03 filter started...\n";

   while (<>)
   {
      my ($ID, $Command, $File) = split / |\r|\n/;

      print "$ID OK\n"      and next if ($Command =~ /^quit$/i);
	   print "$ID INTF 3\n"  and next if ($Command =~ /^intf$/i);
	   print "$ID OK\n"      and next if ($Command =~ /^key$/i);
		print "$ID FAILURE\n" and next if ($Command !~ /^file$/i); # other unsupported commands

      if (open(FILE, $File))
      {
         my $line = <FILE>; chomp($line);
         while($line ne '')
         {
            $line = <FILE>; chomp($line);
         }

         my $mtime = (stat $configfile)[9];
         if ($mtime != $Filemod)
         {
            print "* Config file modified, reloading it\n";
            $Filemod = $mtime;
            LoadConfig($configfile);
         }

         my %matchinglines = ();
         my $count = 0;
         $line = <FILE>; chomp($line);
         while((defined $line) and ($line ne ''))
         {
            foreach my $regex (keys %RegexpsHead)
            {
               if (eval("\$line =~ $regex"))
               {
                  $matchinglines{$RegexpsHead{$regex}} = 1;
                  $count++;
               }
            }
            foreach my $regex (keys %RegexpsFull)
            {
               if (eval("\$line =~ $regex"))
               {
                  $matchinglines{$RegexpsFull{$regex}} = 1;
                  $count++;
               }
            }
            $line = <FILE>; chomp($line);
         }

         my $linecount = 0;
         $line = <FILE>; chomp($line);
         while(defined $line || ($linecount++ <= $MaxBodyLines))
         {
            foreach my $regex (keys %RegexpsBody)
            {
               if (eval("\$line =~ $regex"))
               {
                  $matchinglines{$RegexpsBody{$regex}} = 1;
                  $count++;
               }
            }
            foreach my $regex (keys %RegexpsFull)
            {
               if (eval("\$line =~ $regex"))
               {
                  $matchinglines{$RegexpsFull{$regex}} = 1;
                  $count++;
               }
            }
            $line = <FILE>; chomp($line);
         }
         close FILE;

         my @headers;

         if ($count > 0)
         {
            print "* Found $count matching lines(s)\n";
         }
         else
         {
            print "* Found no matching line\n";
         }

         foreach (keys %matchinglines)
         {
            push @headers, "X-Regex-Match: $_";
         }
         push @headers, "X-ExtScanner: Niversoft's regex_matcher (free)";
         print "$ID ADDHEADER \"".Stringify(@headers)."\"\n";
      }
      else
      {
            print "* Error file not found\n";
            print "$ID OK\n";
      }
      open STDOUT, ">&STDOUT";
   }
}

